@@ -20,7 +20,7 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
- 
+
 #include <config.h>
 #include <dbus/dbus-internals.h>
 #include <string.h>
@@ -49,7 +49,7 @@ bus_config_parser_element_name_to_type (const char *name)
     }
   else if (strcmp (name, "type") == 0)
     {
-      return ELEMENT_TYPE;
+      return ELEMENT_CONFIGTYPE;
     }
   else if (strcmp (name, "fork") == 0)
     {
@@ -127,6 +127,10 @@ bus_config_parser_element_name_to_type (const char *name)
     {
       return ELEMENT_ALLOW_ANONYMOUS;
     }
+  else if (strcmp (name, "apparmor") == 0)
+    {
+      return ELEMENT_APPARMOR;
+    }
   return ELEMENT_NONE;
 }
 
@@ -169,7 +173,7 @@ bus_config_parser_element_type_to_name (ElementType type)
       return "servicehelper";
     case ELEMENT_INCLUDEDIR:
       return "includedir";
-    case ELEMENT_TYPE:
+    case ELEMENT_CONFIGTYPE:
       return "type";
     case ELEMENT_SELINUX:
       return "selinux";
@@ -181,10 +185,11 @@ bus_config_parser_element_type_to_name (ElementType type)
       return "keep_umask";
     case ELEMENT_ALLOW_ANONYMOUS:
       return "allow_anonymous";
+    case ELEMENT_APPARMOR:
+      return "apparmor";
+    default:
+      _dbus_assert_not_reached ("bad element type");
+      return NULL;
     }
-
-  _dbus_assert_not_reached ("bad element type");
-
-  return NULL;
 }
 
