@@ -4,7 +4,7 @@
  * Copyright (C) 2003, 2004  Red Hat, Inc.
  *
  * Licensed under the Academic Free License version 2.1
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -14,7 +14,7 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
@@ -29,6 +29,7 @@
 #include <dbus/dbus-list.h>
 #include <dbus/dbus-hash.h>
 #include <dbus/dbus-internals.h>
+#include <dbus/dbus-message-internal.h>
 
 BusPolicyRule*
 bus_policy_rule_new (BusPolicyRuleType type,
@@ -69,6 +70,8 @@ bus_policy_rule_new (BusPolicyRuleType type,
       break;
     case BUS_POLICY_RULE_OWN:
       break;
+    default:
+      _dbus_assert_not_reached ("invalid rule");
     }
   
   return rule;
@@ -116,6 +119,8 @@ bus_policy_rule_unref (BusPolicyRule *rule)
           break;
         case BUS_POLICY_RULE_GROUP:
           break;
+        default:
+          _dbus_assert_not_reached ("invalid rule");
         }
       
       dbus_free (rule);
@@ -261,6 +266,9 @@ add_list_to_client (DBusList        **list,
           if (!bus_client_policy_append_rule (client, rule))
             return FALSE;
           break;
+
+        default:
+          _dbus_assert_not_reached ("invalid rule");
         }
     }
   
@@ -830,8 +838,11 @@ bus_client_policy_optimize (BusClientPolicy *policy)
           remove_preceding =
             rule->d.own.service_name == NULL;
           break;
+
+        /* The other rule types don't appear in this list */
         case BUS_POLICY_RULE_USER:
         case BUS_POLICY_RULE_GROUP:
+        default:
           _dbus_assert_not_reached ("invalid rule");
           break;
         }
@@ -986,13 +997,38 @@ bus_client_policy_check_can_send (BusClientPolicy *policy,
               continue;
             }
         }
-      
-      if (rule->d.send.destination != NULL)
+
+      if (rule->d.send.broadcast != BUS_POLICY_TRISTATE_ANY)
+        {
+          if (dbus_message_get_destination (message) == NULL &&
+              dbus_message_get_type (message) == DBUS_MESSAGE_TYPE_SIGNAL)
+            {
+              /* it's a broadcast */
+              if (rule->d.send.broadcast == BUS_POLICY_TRISTATE_FALSE)
+                {
+                  _dbus_verbose ("  (policy) skipping rule because message is a broadcast\n");
+                  continue;
+                }
+            }
+          /* else it isn't a broadcast: there is some destination */
+          else if (rule->d.send.broadcast == BUS_POLICY_TRISTATE_TRUE)
+            {
+              _dbus_verbose ("  (policy) skipping rule because message is not a broadcast\n");
+              continue;
+            }
+        }
+
+      if (rule->d.send.destination != NULL && !rule->d.send.destination_is_prefix)
         {
           /* receiver can be NULL for messages that are sent to the
            * message bus itself, we check the strings in that case as
            * built-in services don't have a DBusConnection but messages
            * to them have a destination service name.
+           *
+           * Similarly, receiver can be NULL when we're deciding whether
+           * activation should be allowed; we make the authorization decision
+           * on the assumption that the activated service will have the
+           * requested name and no others.
            */
           if (receiver == NULL)
             {
@@ -1008,9 +1044,9 @@ bus_client_policy_check_can_send (BusClientPolicy *policy,
             {
               DBusString str;
               BusService *service;
-              
+
               _dbus_string_init_const (&str, rule->d.send.destination);
-              
+
               service = bus_registry_lookup (registry, &str);
               if (service == NULL)
                 {
@@ -1019,15 +1055,66 @@ bus_client_policy_check_can_send (BusClientPolicy *policy,
                   continue;
                 }
 
-              if (!bus_service_has_owner (service, receiver))
+              if (!bus_service_owner_in_queue (service, receiver))
                 {
-                  _dbus_verbose ("  (policy) skipping rule because dest %s isn't owned by receiver\n",
+                  _dbus_verbose ("  (policy) skipping rule because receiver isn't primary or queued owner of name %s\n",
                                  rule->d.send.destination);
                   continue;
                 }
             }
         }
 
+      if (rule->d.send.destination != NULL && rule->d.send.destination_is_prefix)
+        {
+          /* receiver can be NULL - the same as in !send.destination_is_prefix */
+          if (receiver == NULL)
+            {
+              const char *destination = dbus_message_get_destination (message);
+              DBusString dest_name;
+
+              if (destination == NULL)
+                {
+                  _dbus_verbose ("  (policy) skipping rule because message has no dest\n");
+                  continue;
+                }
+
+              _dbus_string_init_const (&dest_name, destination);
+
+              if (!_dbus_string_starts_with_words_c_str (&dest_name,
+                                                         rule->d.send.destination,
+                                                         '.'))
+                {
+                  _dbus_verbose ("  (policy) skipping rule because message dest doesn't have prefix %s\n",
+                                 rule->d.send.destination);
+                  continue;
+                }
+            }
+          else
+            {
+              if (!bus_connection_is_queued_owner_by_prefix (receiver,
+                                                             rule->d.send.destination))
+                {
+                  _dbus_verbose ("  (policy) skipping rule because recipient isn't primary or queued owner of any name below %s\n",
+                                 rule->d.send.destination);
+                  continue;
+                }
+            }
+        }
+
+      if (rule->d.send.min_fds > 0 ||
+          rule->d.send.max_fds < DBUS_MAXIMUM_MESSAGE_UNIX_FDS)
+        {
+          unsigned int n_fds = _dbus_message_get_n_unix_fds (message);
+
+          if (n_fds < rule->d.send.min_fds || n_fds > rule->d.send.max_fds)
+            {
+              _dbus_verbose ("  (policy) skipping rule because message has %u fds "
+                             "and that is outside range [%u,%u]",
+                             n_fds, rule->d.send.min_fds, rule->d.send.max_fds);
+              continue;
+            }
+        }
+
       /* Use this rule */
       allowed = rule->allow;
       *log = rule->d.send.log;
@@ -1220,15 +1307,30 @@ bus_client_policy_check_can_receive (BusClientPolicy *policy,
                   continue;
                 }
 
-              if (!bus_service_has_owner (service, sender))
+              if (!bus_service_owner_in_queue (service, sender))
                 {
-                  _dbus_verbose ("  (policy) skipping rule because origin %s isn't owned by sender\n",
+                  _dbus_verbose ("  (policy) skipping rule because sender isn't primary or queued owner of %s\n",
                                  rule->d.receive.origin);
                   continue;
                 }
             }
         }
-      
+
+      if (rule->d.receive.min_fds > 0 ||
+          rule->d.receive.max_fds < DBUS_MAXIMUM_MESSAGE_UNIX_FDS)
+        {
+          unsigned int n_fds = _dbus_message_get_n_unix_fds (message);
+
+          if (n_fds < rule->d.receive.min_fds || n_fds > rule->d.receive.max_fds)
+            {
+              _dbus_verbose ("  (policy) skipping rule because message has %u fds "
+                             "and that is outside range [%u,%u]",
+                             n_fds, rule->d.receive.min_fds,
+                             rule->d.receive.max_fds);
+              continue;
+            }
+        }
+
       /* Use this rule */
       allowed = rule->allow;
       (*toggles)++;
@@ -1240,25 +1342,26 @@ bus_client_policy_check_can_receive (BusClientPolicy *policy,
   return allowed;
 }
 
-dbus_bool_t
-bus_client_policy_check_can_own (BusClientPolicy  *policy,
-                                 DBusConnection   *connection,
-                                 const DBusString *service_name)
+
+
+static dbus_bool_t
+bus_rules_check_can_own (DBusList *rules,
+                         const DBusString *service_name)
 {
   DBusList *link;
   dbus_bool_t allowed;
   
-  /* policy->rules is in the order the rules appeared
+  /* rules is in the order the rules appeared
    * in the config file, i.e. last rule that applies wins
    */
 
   allowed = FALSE;
-  link = _dbus_list_get_first_link (&policy->rules);
+  link = _dbus_list_get_first_link (&rules);
   while (link != NULL)
     {
       BusPolicyRule *rule = link->data;
 
-      link = _dbus_list_get_next_link (&policy->rules, link);
+      link = _dbus_list_get_next_link (&rules, link);
       
       /* Rule is skipped if it specifies a different service name from
        * the desired one.
@@ -1267,12 +1370,19 @@ bus_client_policy_check_can_own (BusClientPolicy  *policy,
       if (rule->type != BUS_POLICY_RULE_OWN)
         continue;
 
-      if (rule->d.own.service_name != NULL)
+      if (!rule->d.own.prefix && rule->d.own.service_name != NULL)
         {
           if (!_dbus_string_equal_c_str (service_name,
                                          rule->d.own.service_name))
             continue;
         }
+      else if (rule->d.own.prefix)
+        {
+          if (!_dbus_string_starts_with_words_c_str (service_name,
+                                                     rule->d.own.service_name,
+                                                     '.'))
+            continue;
+        }
 
       /* Use this rule */
       allowed = rule->allow;
@@ -1281,17 +1391,19 @@ bus_client_policy_check_can_own (BusClientPolicy  *policy,
   return allowed;
 }
 
-#ifdef DBUS_BUILD_TESTS
-
 dbus_bool_t
-bus_policy_test (const DBusString *test_data_dir)
+bus_client_policy_check_can_own (BusClientPolicy  *policy,
+                                 const DBusString *service_name)
 {
-  /* This doesn't do anything for now because I decided to do it in
-   * dispatch.c instead by having some of the clients in dispatch.c
-   * have particular policies applied to them.
-   */
-  
-  return TRUE;
+  return bus_rules_check_can_own (policy->rules, service_name);
 }
 
-#endif /* DBUS_BUILD_TESTS */
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
+dbus_bool_t
+bus_policy_check_can_own (BusPolicy  *policy,
+                          const DBusString *service_name)
+{
+  return bus_rules_check_can_own (policy->default_rules, service_name);
+}
+#endif /* DBUS_ENABLE_EMBEDDED_TESTS */
+
