@@ -5,7 +5,7 @@
  * Copyright (C) 2004,2005  Red Hat, Inc.
  *
  * Licensed under the Academic Free License version 2.1
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -15,7 +15,7 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
@@ -698,7 +698,7 @@ static const char* invalid_escaped_values[] = {
 };
 
 dbus_bool_t
-_dbus_address_test (void)
+_dbus_address_test (const char *test_data_dir _DBUS_GNUC_UNUSED)
 {
   DBusAddressEntry **entries;
   int len;
