@@ -4,7 +4,7 @@
  * Copyright (C) 2002  Red Hat Inc.
  *
  * Licensed under the Academic Free License version 2.1
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -14,7 +14,7 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
@@ -25,6 +25,7 @@
 
 #include <dbus/dbus-internals.h>
 #include <dbus/dbus-connection.h>
+#include <dbus/dbus-credentials.h>
 #include <dbus/dbus-message.h>
 #include <dbus/dbus-transport.h>
 #include <dbus/dbus-resources.h>
@@ -44,9 +45,15 @@ typedef enum
 /** default timeout value when waiting for a message reply, 25 seconds */
 #define _DBUS_DEFAULT_TIMEOUT_VALUE (25 * 1000)
 
+typedef void (* DBusPendingFdsChangeFunction) (void *data);
+
+DBUS_PRIVATE_EXPORT
 void              _dbus_connection_lock                        (DBusConnection     *connection);
+DBUS_PRIVATE_EXPORT
 void              _dbus_connection_unlock                      (DBusConnection     *connection);
+DBUS_PRIVATE_EXPORT
 DBusConnection *  _dbus_connection_ref_unlocked                (DBusConnection     *connection);
+DBUS_PRIVATE_EXPORT
 void              _dbus_connection_unref_unlocked              (DBusConnection     *connection);
 void              _dbus_connection_queue_received_message_link (DBusConnection     *connection,
                                                                 DBusList           *link);
@@ -94,14 +101,28 @@ dbus_bool_t       _dbus_connection_send_and_unlock             (DBusConnection
 
 void              _dbus_connection_queue_synthesized_message_link (DBusConnection *connection,
 						                   DBusList *link);
+DBUS_PRIVATE_EXPORT
 void              _dbus_connection_test_get_locks                 (DBusConnection *conn,
                                                                    DBusMutex **mutex_loc,
                                                                    DBusMutex **dispatch_mutex_loc,
                                                                    DBusMutex **io_path_mutex_loc,
                                                                    DBusCondVar **dispatch_cond_loc,
                                                                    DBusCondVar **io_path_cond_loc);
+DBUS_PRIVATE_EXPORT
+int               _dbus_connection_get_pending_fds_count          (DBusConnection *connection);
+DBUS_PRIVATE_EXPORT
+void              _dbus_connection_set_pending_fds_function       (DBusConnection *connection,
+                                                                   DBusPendingFdsChangeFunction callback,
+                                                                   void *data);
+
+DBUS_PRIVATE_EXPORT
+dbus_bool_t       _dbus_connection_get_linux_security_label       (DBusConnection  *connection,
+                                                                   char           **label_p);
+DBUS_PRIVATE_EXPORT
+DBusCredentials  *_dbus_connection_get_credentials                (DBusConnection  *connection);
 
 /* if DBUS_ENABLE_STATS */
+DBUS_PRIVATE_EXPORT
 void _dbus_connection_get_stats (DBusConnection *connection,
                                  dbus_uint32_t  *in_messages,
                                  dbus_uint32_t  *in_bytes,
@@ -115,7 +136,8 @@ void _dbus_connection_get_stats (DBusConnection *connection,
                                  dbus_uint32_t  *out_peak_fds);
 
 
-/* if DBUS_BUILD_TESTS */
+/* if DBUS_ENABLE_EMBEDDED_TESTS */
+DBUS_PRIVATE_EXPORT
 const char* _dbus_connection_get_address (DBusConnection *connection);
 
 /* This _dbus_bus_* stuff doesn't really belong here, but dbus-bus-internal.h seems
