@@ -4,7 +4,7 @@
  * Copyright (C) 2002, 2003  Red Hat Inc.
  *
  * Licensed under the Academic Free License version 2.1
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -14,7 +14,7 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
@@ -26,6 +26,8 @@
 #include "dbus-internals.h"
 #include "dbus-sysdeps.h"
 #include "dbus-list.h"
+#include "dbus-threads.h"
+#include <dbus/dbus-test-tap.h>
 #include <stdlib.h>
 
 /**
@@ -96,7 +98,7 @@
  * @{
  */
 
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
 static dbus_bool_t debug_initialized = FALSE;
 static int fail_nth = -1;
 static size_t fail_size = 0;
@@ -239,7 +241,7 @@ _dbus_get_fail_alloc_failures (void)
   return n_failures_per_failure;
 }
 
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
 /**
  * Called when about to alloc some memory; if
  * it returns #TRUE, then the allocation should
@@ -252,7 +254,7 @@ dbus_bool_t
 _dbus_decrement_fail_alloc_counter (void)
 {
   _dbus_initialize_malloc_debug ();
-#ifdef DBUS_WIN_FIXME
+#ifdef DBUS_WIN
   {
     static dbus_bool_t called = 0;
 
@@ -293,7 +295,7 @@ _dbus_decrement_fail_alloc_counter (void)
       return FALSE;
     }
 }
-#endif /* DBUS_BUILD_TESTS */
+#endif /* DBUS_ENABLE_EMBEDDED_TESTS */
 
 /**
  * Get the number of outstanding malloc()'d blocks.
@@ -333,9 +335,10 @@ source_string (BlockSource source)
       return "malloc0";
     case SOURCE_REALLOC_NULL:
       return "realloc(NULL)";
+    default:
+      _dbus_assert_not_reached ("Invalid malloc block source ID");
+      return "invalid!";
     }
-  _dbus_assert_not_reached ("Invalid malloc block source ID");
-  return "invalid!";
 }
 
 static void
@@ -363,7 +366,7 @@ check_guards (void       *free_block,
           dbus_uint32_t value = *(dbus_uint32_t*) &block[i];
           if (value != GUARD_VALUE)
             {
-              _dbus_warn ("Block of %lu bytes from %s had start guard value 0x%ux at %d expected 0x%x\n",
+              _dbus_warn ("Block of %lu bytes from %s had start guard value 0x%ux at %d expected 0x%x",
                           (long) requested_bytes, source_string (source),
                           value, i, GUARD_VALUE);
               failed = TRUE;
@@ -378,7 +381,7 @@ check_guards (void       *free_block,
           dbus_uint32_t value = *(dbus_uint32_t*) &block[i];
           if (value != GUARD_VALUE)
             {
-              _dbus_warn ("Block of %lu bytes from %s had end guard value 0x%ux at %d expected 0x%x\n",
+              _dbus_warn ("Block of %lu bytes from %s had end guard value 0x%ux at %d expected 0x%x",
                           (long) requested_bytes, source_string (source),
                           value, i, GUARD_VALUE);
               failed = TRUE;
@@ -459,7 +462,7 @@ set_guards (void       *real_block,
 void*
 dbus_malloc (size_t bytes)
 {
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
   _dbus_initialize_malloc_debug ();
   
   if (_dbus_decrement_fail_alloc_counter ())
@@ -471,7 +474,7 @@ dbus_malloc (size_t bytes)
 
   if (bytes == 0) /* some system mallocs handle this, some don't */
     return NULL;
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
   else if (fail_size != 0 && bytes > fail_size)
     return NULL;
   else if (guards)
@@ -485,7 +488,7 @@ dbus_malloc (size_t bytes)
         }
       else if (malloc_cannot_fail)
         {
-          _dbus_warn ("out of memory: malloc (%ld + %ld)\n",
+          _dbus_warn ("out of memory: malloc (%ld + %ld)",
               (long) bytes, (long) GUARD_EXTRA_SIZE);
           _dbus_abort ();
         }
@@ -498,14 +501,14 @@ dbus_malloc (size_t bytes)
       void *mem;
       mem = malloc (bytes);
 
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
       if (mem)
         {
           _dbus_atomic_inc (&n_blocks_outstanding);
         }
       else if (malloc_cannot_fail)
         {
-          _dbus_warn ("out of memory: malloc (%ld)\n", (long) bytes);
+          _dbus_warn ("out of memory: malloc (%ld)", (long) bytes);
           _dbus_abort ();
         }
 #endif
@@ -529,7 +532,7 @@ dbus_malloc (size_t bytes)
 void*
 dbus_malloc0 (size_t bytes)
 {
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
   _dbus_initialize_malloc_debug ();
   
   if (_dbus_decrement_fail_alloc_counter ())
@@ -542,7 +545,7 @@ dbus_malloc0 (size_t bytes)
   
   if (bytes == 0)
     return NULL;
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
   else if (fail_size != 0 && bytes > fail_size)
     return NULL;
   else if (guards)
@@ -557,7 +560,7 @@ dbus_malloc0 (size_t bytes)
         }
       else if (malloc_cannot_fail)
         {
-          _dbus_warn ("out of memory: calloc (%ld + %ld, 1)\n",
+          _dbus_warn ("out of memory: calloc (%ld + %ld, 1)",
               (long) bytes, (long) GUARD_EXTRA_SIZE);
           _dbus_abort ();
         }
@@ -570,14 +573,14 @@ dbus_malloc0 (size_t bytes)
       void *mem;
       mem = calloc (bytes, 1);
 
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
       if (mem)
         {
           _dbus_atomic_inc (&n_blocks_outstanding);
         }
       else if (malloc_cannot_fail)
         {
-          _dbus_warn ("out of memory: calloc (%ld)\n", (long) bytes);
+          _dbus_warn ("out of memory: calloc (%ld)", (long) bytes);
           _dbus_abort ();
         }
 #endif
@@ -600,7 +603,7 @@ void*
 dbus_realloc (void  *memory,
               size_t bytes)
 {
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
   _dbus_initialize_malloc_debug ();
   
   if (_dbus_decrement_fail_alloc_counter ())
@@ -616,7 +619,7 @@ dbus_realloc (void  *memory,
       dbus_free (memory);
       return NULL;
     }
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
   else if (fail_size != 0 && bytes > fail_size)
     return NULL;
   else if (guards)
@@ -635,7 +638,7 @@ dbus_realloc (void  *memory,
             {
               if (malloc_cannot_fail)
                 {
-                  _dbus_warn ("out of memory: realloc (%p, %ld + %ld)\n",
+                  _dbus_warn ("out of memory: realloc (%p, %ld + %ld)",
                       memory, (long) bytes, (long) GUARD_EXTRA_SIZE);
                   _dbus_abort ();
                 }
@@ -662,7 +665,7 @@ dbus_realloc (void  *memory,
             }
           else if (malloc_cannot_fail)
             {
-              _dbus_warn ("out of memory: malloc (%ld + %ld)\n",
+              _dbus_warn ("out of memory: malloc (%ld + %ld)",
                   (long) bytes, (long) GUARD_EXTRA_SIZE);
               _dbus_abort ();
             }
@@ -676,10 +679,10 @@ dbus_realloc (void  *memory,
       void *mem;
       mem = realloc (memory, bytes);
 
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
       if (mem == NULL && malloc_cannot_fail)
         {
-          _dbus_warn ("out of memory: malloc (%ld)\n", (long) bytes);
+          _dbus_warn ("out of memory: malloc (%ld)", (long) bytes);
           _dbus_abort ();
         }
 
@@ -699,7 +702,7 @@ dbus_realloc (void  *memory,
 void
 dbus_free (void  *memory)
 {
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
   if (guards)
     {
       check_guards (memory, TRUE);
@@ -723,7 +726,7 @@ dbus_free (void  *memory)
     
   if (memory) /* we guarantee it's safe to free (NULL) */
     {
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
 #ifdef DBUS_DISABLE_ASSERT
       _dbus_atomic_dec (&n_blocks_outstanding);
 #else
@@ -794,7 +797,7 @@ struct ShutdownClosure
   void *data;                /**< Data for function */
 };
 
-_DBUS_DEFINE_GLOBAL_LOCK (shutdown_funcs);
+/* Protected by _DBUS_LOCK (shutdown_funcs) */
 static ShutdownClosure *registered_globals = NULL;
 
 /**
@@ -808,6 +811,20 @@ static ShutdownClosure *registered_globals = NULL;
 dbus_bool_t
 _dbus_register_shutdown_func (DBusShutdownFunction  func,
                               void                 *data)
+{
+  dbus_bool_t ok;
+
+  if (!_DBUS_LOCK (shutdown_funcs))
+    return FALSE;
+
+  ok = _dbus_register_shutdown_func_unlocked (func, data);
+  _DBUS_UNLOCK (shutdown_funcs);
+  return ok;
+}
+
+dbus_bool_t
+_dbus_register_shutdown_func_unlocked (DBusShutdownFunction  func,
+                                       void                 *data)
 {
   ShutdownClosure *c;
 
@@ -819,13 +836,9 @@ _dbus_register_shutdown_func (DBusShutdownFunction  func,
   c->func = func;
   c->data = data;
 
-  _DBUS_LOCK (shutdown_funcs);
-  
   c->next = registered_globals;
   registered_globals = c;
 
-  _DBUS_UNLOCK (shutdown_funcs);
-  
   return TRUE;
 }
 
@@ -845,12 +858,19 @@ _dbus_register_shutdown_func (DBusShutdownFunction  func,
  * can be useful to free these internal data structures.
  *
  * dbus_shutdown() does NOT free memory that was returned
- * to the application. It only returns libdbus-internal
+ * to the application. It only frees libdbus-internal
  * data structures.
  *
  * You MUST free all memory and release all reference counts
  * returned to you by libdbus prior to calling dbus_shutdown().
  *
+ * If a shared connection is open, calling dbus_shutdown() will
+ * drain its queue of messages and disconnect it. In particular,
+ * this will result in processing of the special Disconnected
+ * signal, which may result in a call to _exit(), unless you
+ * have used dbus_connection_set_exit_on_disconnect() to disable
+ * that behaviour.
+ *
  * You can't continue to use any D-Bus objects, such as connections,
  * that were allocated prior to dbus_shutdown(). You can, however,
  * start over; call dbus_threads_init() again, create new connections,
@@ -890,12 +910,18 @@ dbus_shutdown (void)
       dbus_free (c);
     }
 
+  /* We wrap this in the thread-initialization lock because
+   * dbus_threads_init() uses the current generation to tell whether
+   * we're initialized, so we need to make sure that un-initializing
+   * propagates into all threads. */
+  _dbus_threads_lock_platform_specific ();
   _dbus_current_generation += 1;
+  _dbus_threads_unlock_platform_specific ();
 }
 
 /** @} */ /** End of public API docs block */
 
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
 #include "dbus-test.h"
 
 /**
@@ -904,7 +930,7 @@ dbus_shutdown (void)
  * @returns #TRUE on success.
  */
 dbus_bool_t
-_dbus_memory_test (void)
+_dbus_memory_test (const char *test_data_dir _DBUS_GNUC_UNUSED)
 {
   dbus_bool_t old_guards;
   void *p;
@@ -914,18 +940,18 @@ _dbus_memory_test (void)
   guards = TRUE;
   p = dbus_malloc (4);
   if (p == NULL)
-    _dbus_assert_not_reached ("no memory");
+    _dbus_test_fatal ("no memory");
   for (size = 4; size < 256; size += 4)
     {
       p = dbus_realloc (p, size);
       if (p == NULL)
-	_dbus_assert_not_reached ("no memory");
+        _dbus_test_fatal ("no memory");
     }
   for (size = 256; size != 0; size -= 4)
     {
       p = dbus_realloc (p, size);
       if (p == NULL)
-	_dbus_assert_not_reached ("no memory");
+        _dbus_test_fatal ("no memory");
     }
   dbus_free (p);
   guards = old_guards;
