@@ -22,9 +22,11 @@
  *
  */
 
+#include <config.h>
 #include "dbus-internals.h"
 #include "dbus-sha.h"
 #include "dbus-marshal-basic.h" /* for byteswap routines */
+#include <dbus/dbus-test-tap.h>
 #include <string.h>
 
 /* The following comments have the history of where this code
@@ -459,13 +461,13 @@ _dbus_sha_final (DBusSHAContext   *context,
 
   sha_finish (context, digest);
 
-  if (!_dbus_string_append_len (results, digest, 20))
+  if (!_dbus_string_append_len (results, (const char *) digest, 20))
     return FALSE;
 
   /* some kind of security paranoia, though it seems pointless
    * to me given the nonzeroed stuff flying around
    */
-  memset ((void*)context, '\0', sizeof (DBusSHAContext));
+  _DBUS_ZERO(*context);
 
   return TRUE;
 }
@@ -510,7 +512,7 @@ _dbus_sha_compute (const DBusString *data,
 
 /** @} */ /* end of exported functions */
 
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
 #include "dbus-test.h"
 #include <stdio.h>
 
@@ -523,18 +525,18 @@ check_sha_binary (const unsigned char *input,
   DBusString expected_str;
   DBusString results;
 
-  _dbus_string_init_const_len (&input_str, input, input_len);
+  _dbus_string_init_const_len (&input_str, (const char *) input, input_len);
   _dbus_string_init_const (&expected_str, expected);
 
   if (!_dbus_string_init (&results))
-    _dbus_assert_not_reached ("no memory for SHA-1 results");
+    _dbus_test_fatal ("no memory for SHA-1 results");
 
   if (!_dbus_sha_compute (&input_str, &results))
-    _dbus_assert_not_reached ("no memory for SHA-1 results");
+    _dbus_test_fatal ("no memory for SHA-1 results");
 
   if (!_dbus_string_equal (&expected_str, &results))
     {
-      _dbus_warn ("Expected hash %s got %s for SHA-1 sum\n",
+      _dbus_warn ("Expected hash %s got %s for SHA-1 sum",
                   expected,
                   _dbus_string_get_const_data (&results));
       _dbus_string_free (&results);
@@ -549,7 +551,7 @@ static dbus_bool_t
 check_sha_str (const char *input,
                const char *expected)
 {
-  return check_sha_binary (input, strlen (input), expected);
+  return check_sha_binary ((unsigned char *) input, strlen (input), expected);
 }
 
 static dbus_bool_t
@@ -620,14 +622,14 @@ decode_compact_string (const DBusString *line,
           if (byte_containing_next_bit >= _dbus_string_get_length (decoded))
             {
               if (!_dbus_string_set_length (decoded, byte_containing_next_bit + 1))
-                _dbus_assert_not_reached ("no memory to extend to next byte");
+                _dbus_test_fatal ("no memory to extend to next byte");
             }
 
           old_byte = _dbus_string_get_byte (decoded, byte_containing_next_bit);
           old_byte |= current_b << bit_containing_next_bit;
 
 #if 0
-          printf ("Appending bit %d to byte %d at bit %d resulting in byte 0x%x\n",
+          _dbus_test_diag ("Appending bit %d to byte %d at bit %d resulting in byte 0x%x",
                   current_b, byte_containing_next_bit,
                   bit_containing_next_bit, old_byte);
 #endif
@@ -664,7 +666,7 @@ get_next_expected_result (DBusString *results,
   retval = FALSE;
   
   if (!_dbus_string_init (&line))
-    _dbus_assert_not_reached ("no memory");
+    _dbus_test_fatal ("no memory");
   
  next_iteration:
   while (_dbus_string_pop_line (results, &line))
@@ -687,12 +689,14 @@ get_next_expected_result (DBusString *results,
           int i;
           
           if (!_dbus_string_move (&line, 0, result, 0))
-            _dbus_assert_not_reached ("no memory");
+            _dbus_test_fatal ("no memory");
 
           i = 0;
           while (i < _dbus_string_get_length (result))
             {
-              switch (_dbus_string_get_byte (result, i))
+              unsigned char c = _dbus_string_get_byte (result, i);
+
+              switch (c)
                 {
                 case 'A':
                   _dbus_string_set_byte (result, i, 'a');
@@ -717,6 +721,9 @@ get_next_expected_result (DBusString *results,
                   _dbus_string_delete (result, i, 1);
                   --i; /* to offset ++i below */
                   break;
+                default:
+                  if ((c < '0' || c > '9') && (c < 'a' || c > 'f'))
+                    _dbus_test_fatal ("invalid SHA-1 test script");
                 }
 
               ++i;
@@ -750,33 +757,33 @@ process_test_data (const char *test_data_dir)
   retval = FALSE;
   
   if (!_dbus_string_init (&tests_file))
-    _dbus_assert_not_reached ("no memory");
+    _dbus_test_fatal ("no memory");
 
   if (!_dbus_string_init (&results_file))
-    _dbus_assert_not_reached ("no memory");
+    _dbus_test_fatal ("no memory");
 
   if (!_dbus_string_init (&tests))
-    _dbus_assert_not_reached ("no memory");
+    _dbus_test_fatal ("no memory");
 
   if (!_dbus_string_init (&results))
-    _dbus_assert_not_reached ("no memory");
+    _dbus_test_fatal ("no memory");
 
   if (!_dbus_string_init (&line))
-    _dbus_assert_not_reached ("no memory");
-  
+    _dbus_test_fatal ("no memory");
+
   if (!_dbus_string_append (&tests_file, test_data_dir))
-    _dbus_assert_not_reached ("no memory");
+    _dbus_test_fatal ("no memory");
 
   if (!_dbus_string_append (&results_file, test_data_dir))
-    _dbus_assert_not_reached ("no memory");
+    _dbus_test_fatal ("no memory");
 
   _dbus_string_init_const (&tmp, "sha-1/byte-messages.sha1");
   if (!_dbus_concat_dir_and_file (&tests_file, &tmp))
-    _dbus_assert_not_reached ("no memory");
+    _dbus_test_fatal ("no memory");
 
   _dbus_string_init_const (&tmp, "sha-1/byte-hashes.sha1");
   if (!_dbus_concat_dir_and_file (&results_file, &tmp))
-    _dbus_assert_not_reached ("no memory");
+    _dbus_test_fatal ("no memory");
 
   if (!_dbus_file_get_contents (&tests, &tests_file, &error))
     {
@@ -810,7 +817,7 @@ process_test_data (const char *test_data_dir)
         goto next_iteration;
       else if (_dbus_string_starts_with_c_str (&line, "H>"))
         {
-          printf ("SHA-1: %s\n", _dbus_string_get_const_data (&line));
+          _dbus_test_diag ("SHA-1: %s", _dbus_string_get_const_data (&line));
 
           if (_dbus_string_find (&line, 0, "Type 3", NULL))
             {
@@ -820,7 +827,7 @@ process_test_data (const char *test_data_dir)
                * to use those tests.
                */
               
-              printf (" (ending tests due to Type 3 tests seen - this is normal)\n");
+              _dbus_test_diag (" (ending tests due to Type 3 tests seen - this is normal)");
               break;
             }
         }
@@ -838,16 +845,16 @@ process_test_data (const char *test_data_dir)
           success = FALSE;
           
           if (!_dbus_string_init (&next_line))
-            _dbus_assert_not_reached ("no memory");
+            _dbus_test_fatal ("no memory");
 
           if (!_dbus_string_init (&expected))
-            _dbus_assert_not_reached ("no memory");
+            _dbus_test_fatal ("no memory");
           
           if (!_dbus_string_init (&test))
-            _dbus_assert_not_reached ("no memory");
+            _dbus_test_fatal ("no memory");
 
           if (!_dbus_string_init (&result))
-            _dbus_assert_not_reached ("no memory");
+            _dbus_test_fatal ("no memory");
 
           /* the "compact strings" are "^"-terminated not
            * newline-terminated so readahead to find the
@@ -859,7 +866,7 @@ process_test_data (const char *test_data_dir)
               if (!_dbus_string_append_byte (&line, ' ') ||
                   !_dbus_string_move (&next_line, 0, &line,
                                       _dbus_string_get_length (&line)))
-                _dbus_assert_not_reached ("no memory");
+                _dbus_test_fatal ("no memory");
             }
           
           if (!decode_compact_string (&line, &test))
@@ -870,7 +877,7 @@ process_test_data (const char *test_data_dir)
             }
           
           if (!_dbus_sha_compute (&test, &result))
-            _dbus_assert_not_reached ("no memory for SHA-1 result");
+            _dbus_test_fatal ("no memory for SHA-1 result");
 
           if (!get_next_expected_result (&results, &expected))
             {
@@ -906,7 +913,7 @@ process_test_data (const char *test_data_dir)
 
   retval = TRUE;
 
-  printf ("Passed the %d SHA-1 tests in the test file\n",
+  _dbus_test_diag ("Passed the %d SHA-1 tests in the test file",
           success_count);
   
  out:
@@ -937,7 +944,7 @@ _dbus_sha_test (const char *test_data_dir)
         return FALSE;
     }
   else
-    printf ("No test data dir\n");
+    _dbus_test_diag ("No test data dir");
   
   i = 0;
   while (i < 256)
@@ -965,4 +972,4 @@ _dbus_sha_test (const char *test_data_dir)
   return TRUE;
 }
 
-#endif /* DBUS_BUILD_TESTS */
+#endif /* DBUS_ENABLE_EMBEDDED_TESTS */
