@@ -1269,7 +1269,7 @@ dbus_bool_t
 _dbus_spawn_async_with_babysitter (DBusBabysitter          **sitter_p,
                                    const char               *log_name,
                                    char             * const *argv,
-                                   char                    **env,
+                                   char             * const *env,
                                    DBusSpawnFlags            flags,
                                    DBusSpawnChildSetupFunc   child_setup,
                                    void                     *user_data,
@@ -1513,8 +1513,6 @@ _dbus_spawn_async_with_babysitter (DBusBabysitter          **sitter_p,
       else
         _dbus_babysitter_unref (sitter);
 
-      dbus_free_string_array (env);
-
       _DBUS_ASSERT_ERROR_IS_CLEAR (error);
 
       return TRUE;
