@@ -1,10 +1,10 @@
 /* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /* dbus-userdb.c User database abstraction
- * 
+ *
  * Copyright (C) 2003, 2004  Red Hat, Inc.
  *
  * Licensed under the Academic Free License version 2.1
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -14,12 +14,13 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
+#include <config.h>
 #define DBUS_USERDB_INCLUDES_PRIVATE 1
 #include "dbus-userdb.h"
 #include "dbus-hash.h"
@@ -29,6 +30,13 @@
 #include "dbus-credentials.h"
 #include <string.h>
 
+/* It isn't obvious from its name, but this file is part of the Unix
+ * system-dependent part of libdbus. Windows has a parallel
+ * implementation of some of it in dbus-sysdeps-win.c. */
+#if defined(DBUS_WIN) || !defined(DBUS_UNIX)
+#error "This file only makes sense on Unix OSs"
+#endif
+
 /**
  * @addtogroup DBusInternalsUtils
  * @{
@@ -143,9 +151,8 @@ _dbus_user_database_lookup (DBusUserDatabase *db,
         uid = n;
     }
 
-#ifdef DBUS_ENABLE_USERDB_CACHE  
   if (uid != DBUS_UID_UNSET)
-    info = _dbus_hash_table_lookup_ulong (db->users, uid);
+    info = _dbus_hash_table_lookup_uintptr (db->users, uid);
   else
     info = _dbus_hash_table_lookup_string (db->users_by_name, _dbus_string_get_const_data (username));
 
@@ -156,9 +163,6 @@ _dbus_user_database_lookup (DBusUserDatabase *db,
       return info;
     }
   else
-#else 
-  if (1)
-#endif
     {
       if (uid != DBUS_UID_UNSET)
 	_dbus_verbose ("No cache for UID "DBUS_UID_FORMAT"\n",
@@ -198,7 +202,7 @@ _dbus_user_database_lookup (DBusUserDatabase *db,
       username = NULL;
 
       /* insert into hash */
-      if (!_dbus_hash_table_insert_ulong (db->users, info->uid, info))
+      if (!_dbus_hash_table_insert_uintptr (db->users, info->uid, info))
         {
           dbus_set_error (error, DBUS_ERROR_NO_MEMORY, NULL);
           _dbus_user_info_free_allocated (info);
@@ -209,7 +213,7 @@ _dbus_user_database_lookup (DBusUserDatabase *db,
                                            info->username,
                                            info))
         {
-          _dbus_hash_table_remove_ulong (db->users, info->uid);
+          _dbus_hash_table_remove_uintptr (db->users, info->uid);
           dbus_set_error (error, DBUS_ERROR_NO_MEMORY, NULL);
           return NULL;
         }
@@ -226,7 +230,8 @@ static DBusString process_homedir;
 static void
 shutdown_system_db (void *data)
 {
-  _dbus_user_database_unref (system_db);
+  if (system_db != NULL)
+    _dbus_user_database_unref (system_db);
   system_db = NULL;
   _dbus_string_free (&process_username);
   _dbus_string_free (&process_homedir);
@@ -262,7 +267,7 @@ init_system_db (void)
           else
             {
               /* This really should not happen. */
-              _dbus_warn ("Could not get password database information for UID of current process: %s\n",
+              _dbus_warn ("Could not get password database information for UID of current process: %s",
                           error.message);
               dbus_error_free (&error);
               return FALSE;
@@ -304,11 +309,18 @@ init_system_db (void)
 /**
  * Locks global system user database.
  */
-void
+dbus_bool_t
 _dbus_user_database_lock_system (void)
 {
-  _DBUS_LOCK (system_users);
-  database_locked = TRUE;
+  if (_DBUS_LOCK (system_users))
+    {
+      database_locked = TRUE;
+      return TRUE;
+    }
+  else
+    {
+      return FALSE;
+    }
 }
 
 /**
@@ -343,9 +355,14 @@ _dbus_user_database_get_system (void)
 void
 _dbus_user_database_flush_system (void)
 {
-  _dbus_user_database_lock_system ();
-   
-  _dbus_user_database_flush (system_db);
+  if (!_dbus_user_database_lock_system ())
+    {
+      /* nothing to flush */
+      return;
+    }
+
+   if (system_db != NULL)
+    _dbus_user_database_flush (system_db);
 
   _dbus_user_database_unlock_system ();
 }
@@ -360,7 +377,9 @@ _dbus_user_database_flush_system (void)
 dbus_bool_t
 _dbus_username_from_current_process (const DBusString **username)
 {
-  _dbus_user_database_lock_system ();
+  if (!_dbus_user_database_lock_system ())
+    return FALSE;
+
   if (!init_system_db ())
     {
       _dbus_user_database_unlock_system ();
@@ -382,7 +401,9 @@ _dbus_username_from_current_process (const DBusString **username)
 dbus_bool_t
 _dbus_homedir_from_current_process (const DBusString  **homedir)
 {
-  _dbus_user_database_lock_system ();
+  if (!_dbus_user_database_lock_system ())
+    return FALSE;
+
   if (!init_system_db ())
     {
       _dbus_user_database_unlock_system ();
@@ -394,45 +415,6 @@ _dbus_homedir_from_current_process (const DBusString  **homedir)
   return TRUE;
 }
 
-/**
- * Gets the home directory for the given user.
- *
- * @param username the username
- * @param homedir string to append home directory to
- * @returns #TRUE if user existed and we appended their homedir
- */
-dbus_bool_t
-_dbus_homedir_from_username (const DBusString *username,
-                             DBusString       *homedir)
-{
-  DBusUserDatabase *db;
-  const DBusUserInfo *info;
-  _dbus_user_database_lock_system ();
-
-  db = _dbus_user_database_get_system ();
-  if (db == NULL)
-    {
-      _dbus_user_database_unlock_system ();
-      return FALSE;
-    }
-
-  if (!_dbus_user_database_get_username (db, username,
-                                         &info, NULL))
-    {
-      _dbus_user_database_unlock_system ();
-      return FALSE;
-    }
-
-  if (!_dbus_string_append (homedir, info->homedir))
-    {
-      _dbus_user_database_unlock_system ();
-      return FALSE;
-    }
-  
-  _dbus_user_database_unlock_system ();
-  return TRUE;
-}
-
 /**
  * Gets the home directory for the given user.
  *
@@ -446,7 +428,20 @@ _dbus_homedir_from_uid (dbus_uid_t         uid,
 {
   DBusUserDatabase *db;
   const DBusUserInfo *info;
-  _dbus_user_database_lock_system ();
+
+  if (uid == _dbus_getuid () && uid == _dbus_geteuid ())
+    {
+      const char *from_environment;
+
+      from_environment = _dbus_getenv ("HOME");
+
+      if (from_environment != NULL)
+        return _dbus_string_append (homedir, from_environment);
+    }
+
+  /* FIXME: this can't distinguish ENOMEM from other errors */
+  if (!_dbus_user_database_lock_system ())
+    return FALSE;
 
   db = _dbus_user_database_get_system ();
   if (db == NULL)
@@ -487,23 +482,57 @@ _dbus_homedir_from_uid (dbus_uid_t         uid,
  * @returns #TRUE if the username existed and we got some credentials
  */
 dbus_bool_t
-_dbus_credentials_add_from_user (DBusCredentials  *credentials,
-                                 const DBusString *username)
+_dbus_credentials_add_from_user (DBusCredentials         *credentials,
+                                 const DBusString        *username,
+                                 DBusCredentialsAddFlags  flags,
+                                 DBusError               *error)
 {
   DBusUserDatabase *db;
   const DBusUserInfo *info;
+  unsigned long uid = DBUS_UID_UNSET;
 
-  _dbus_user_database_lock_system ();
+  /* Fast-path for the common case: if the "username" is all-numeric,
+   * then it's a Unix uid. This is true regardless of whether that uid
+   * exists in NSS or /etc/passwd or equivalent. */
+  if (_dbus_is_a_number (username, &uid))
+    {
+      _DBUS_STATIC_ASSERT (sizeof (uid) == sizeof (dbus_uid_t));
+
+      if (_dbus_credentials_add_unix_uid (credentials, uid))
+        {
+          return TRUE;
+        }
+      else
+        {
+          _DBUS_SET_OOM (error);
+          return FALSE;
+        }
+    }
+
+  /* If we aren't allowed to look in NSS or /etc/passwd, fail now. */
+  if (!(flags & DBUS_CREDENTIALS_ADD_FLAGS_USER_DATABASE))
+    {
+      dbus_set_error (error, DBUS_ERROR_INVALID_ARGS,
+                      "Expected a numeric Unix uid");
+      return FALSE;
+    }
+
+  if (!_dbus_user_database_lock_system ())
+    {
+      _DBUS_SET_OOM (error);
+      return FALSE;
+    }
 
   db = _dbus_user_database_get_system ();
   if (db == NULL)
     {
       _dbus_user_database_unlock_system ();
+      _DBUS_SET_OOM (error);
       return FALSE;
     }
 
   if (!_dbus_user_database_get_username (db, username,
-                                         &info, NULL))
+                                         &info, error))
     {
       _dbus_user_database_unlock_system ();
       return FALSE;
@@ -512,6 +541,7 @@ _dbus_credentials_add_from_user (DBusCredentials  *credentials,
   if (!_dbus_credentials_add_unix_uid(credentials, info->uid))
     {
       _dbus_user_database_unlock_system ();
+      _DBUS_SET_OOM (error);
       return FALSE;
     }
   
@@ -535,13 +565,13 @@ _dbus_user_database_new (void)
 
   db->refcount = 1;
 
-  db->users = _dbus_hash_table_new (DBUS_HASH_ULONG,
+  db->users = _dbus_hash_table_new (DBUS_HASH_UINTPTR,
                                     NULL, (DBusFreeFunction) _dbus_user_info_free_allocated);
   
   if (db->users == NULL)
     goto failed;
 
-  db->groups = _dbus_hash_table_new (DBUS_HASH_ULONG,
+  db->groups = _dbus_hash_table_new (DBUS_HASH_UINTPTR,
                                      NULL, (DBusFreeFunction) _dbus_group_info_free_allocated);
   
   if (db->groups == NULL)
@@ -576,7 +606,7 @@ _dbus_user_database_flush (DBusUserDatabase *db)
   _dbus_hash_table_remove_all(db->groups);
 }
 
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
 /**
  * Increments refcount of user database.
  * @param db the database
@@ -591,7 +621,7 @@ _dbus_user_database_ref (DBusUserDatabase  *db)
 
   return db;
 }
-#endif /* DBUS_BUILD_TESTS */
+#endif /* DBUS_ENABLE_EMBEDDED_TESTS */
 
 /**
  * Decrements refcount of user database.
